#include <Wire.h>
#include <Adafruit_GFX.h>
#include <Adafruit_SSD1306.h>
#include <SoftwareSerial.h>
#include <DFRobotDFPlayerMini.h>

#define SCREEN_WIDTH 128
#define SCREEN_HEIGHT 64
//#define OLED_RESET -1  // Reset pin not used

#define SSD1306_I2C_ADDRESS   0x3C

Adafruit_SSD1306 display(SCREEN_WIDTH, SCREEN_HEIGHT);

// DFPlayer connections
SoftwareSerial mySerial(7, 8); // RX, TX
DFRobotDFPlayerMini myDFPlayer;

// Button pins
const int buttonPinNext = 10; // Next song button
const int buttonPinPrev = 11; // Previous song button
const int buttonPinSelect = 12; // Select song button
const int buttonPinVolumeUp = 13; // Volume up button
const int buttonPinVolumeDown = 14; // Volume down button

// Song data
const char* songNames[] = {
    "Song 1",  // Change to actual song names
    "Song 2",
    "Song 3",
    // Add more songs as needed
};
const int totalSongs = sizeof(songNames) / sizeof(songNames[0]);
int currentSongIndex = 0; // Start with the first song
int volume = 10; // Initial volume level (0-30)

void setup() {
    // Initialize Serial and the display
    Serial.begin(9600);
    mySerial.begin(9600);
    display.begin(SSD1306_SWITCHCAPVCC, SSD1306_I2C_ADDRESS);
    display.clearDisplay();

    // Initialize DFPlayer
    if (!myDFPlayer.begin(mySerial)) {
        Serial.println("DFPlayer Mini not found");
        while (true);
    }
    myDFPlayer.volume(volume);  // Set initial volume

    // Set up button pins
    pinMode(buttonPinNext, INPUT_PULLUP);
    pinMode(buttonPinPrev, INPUT_PULLUP);
    pinMode(buttonPinSelect, INPUT_PULLUP);
    pinMode(buttonPinVolumeUp, INPUT_PULLUP);
    pinMode(buttonPinVolumeDown, INPUT_PULLUP);

    // Start playing the first song
    myDFPlayer.play(currentSongIndex + 1);  // DFPlayer uses 1-based indexing
}

void loop() {
    // Display song menu
    display.clearDisplay();
    display.setTextSize(1);
    display.setTextColor(SSD1306_WHITE);
    display.setCursor(0, 0);
    display.print("Select Song:");

    // Display the list of songs
    for (int i = 0; i < totalSongs; i++) {
        if (i == currentSongIndex) {
            display.setTextColor(SSD1306_BLACK, SSD1306_WHITE); // Highlight current song
        } else {
            display.setTextColor(SSD1306_WHITE);
        }
        display.setCursor(0, 10 + i * 10);
        display.print(songNames[i]);
    }

    // Display current volume
    display.setTextColor(SSD1306_WHITE);
    display.setCursor(0, 50);
    display.print("Volume: ");
    display.print(volume);
    display.display();

    // Check for button presses
    if (digitalRead(buttonPinNext) == LOW) {
        currentSongIndex = (currentSongIndex + 1) % totalSongs; // Loop back to start
        delay(100); // Simple debounce
    }
    if (digitalRead(buttonPinPrev) == LOW) {
        currentSongIndex = (currentSongIndex - 1 + totalSongs) % totalSongs; // Loop to end
        delay(100); // Simple debounce
    }
    if (digitalRead(buttonPinSelect) == LOW) {
        myDFPlayer.play(currentSongIndex + 1); // Play selected song
        delay(100); // Simple debounce
        playVisualizer();
    }
    if (digitalRead(buttonPinVolumeUp) == LOW) {
        if (volume < 30) { // Max volume is 30
            volume++;
            myDFPlayer.volume(volume); // Update volume on DFPlayer
            displayVolume();
        }
        delay(100); // Simple debounce
    }
    if (digitalRead(buttonPinVolumeDown) == LOW) {
        if (volume > 0) { // Min volume is 0
            volume--;
            myDFPlayer.volume(volume); // Update volume on DFPlayer
            displayVolume();
        }
        delay(100); // Simple debounce
    }
}

// Function to display volume change
void displayVolume() {
    display.clearDisplay();
    display.setTextSize(2);
    display.setTextColor(SSD1306_WHITE);
    display.setCursor(0, 0);
    display.print("Volume: ");
    display.print(volume);
    display.display();
    delay(100); // Show volume for 1 second
}

// Simple visualizer function
void playVisualizer() {
    // Basic visualization while playing (you can expand this)
    display.clearDisplay();
    display.setTextSize(2);
    display.setTextColor(SSD1306_WHITE);
    display.setCursor(0, 0);
    display.print("Now Playing:");
    display.setTextSize(1);
    display.setCursor(0, 20);
    display.print(songNames[currentSongIndex]);
    display.display();
  
    // Visualizer loop (you can customize this)
    for (int i = 0; i < 20; i++) {
        display.fillRect(i * 6, 40, 5, random(10, 30), SSD1306_WHITE);
        display.display();
        delay(50);
        display.fillRect(i * 6, 40, 5, random(10, 30), SSD1306_BLACK);
    }
}